@@ -35,18 +35,22 @@ namespace Twitterizer
     using System;
     using System.Globalization;
     using System.IO;
+    using System.Linq;
     using System.Net;
-    using System.Runtime.Serialization;
-    using System.Security.Permissions;
     using System.Text;
-    using Twitterizer.Core;
+#if !SILVERLIGHT
+    using System.Runtime.Serialization;
+#endif
+    using Core;
 
     /// <summary>
     /// The Twitterizer Exception
     /// </summary>
     /// <seealso cref="System.Net.WebException"/>
+#if !SILVERLIGHT
     [Serializable]
-    public class TwitterizerException : WebException, ISerializable
+#endif
+    public class TwitterizerException : WebException
     {
         #region Constructors
         /// <summary>
@@ -86,19 +90,15 @@ namespace Twitterizer
 
                 byte[] responseData = ConversionUtility.ReadStream(responseStream);
 
-                this.ResponseBody = Encoding.UTF8.GetString(responseData);
+                this.ResponseBody = Encoding.UTF8.GetString(responseData, 0, responseData.Length);
 
-#if DEBUG
-                System.Diagnostics.Debug.WriteLine("----------- RESPONSE -----------");
-                System.Diagnostics.Debug.Write(this.ResponseBody);
-                System.Diagnostics.Debug.WriteLine("----------- END -----------");
-#endif
                 this.ParseRateLimitHeaders(response);
 
                 if (response.ContentType.StartsWith("application/json", StringComparison.OrdinalIgnoreCase))
                 {
                     this.ErrorDetails = SerializationHelper<TwitterErrorDetails>.Deserialize(responseData, null);
                 }
+#if !SILVERLIGHT
                 else if (response.ContentType.StartsWith("text/xml", StringComparison.OrdinalIgnoreCase))
                 {
                     // Try to deserialize as XML (specifically OAuth requests)
@@ -107,20 +107,21 @@ namespace Twitterizer
 
                     this.ErrorDetails = ds.Deserialize(new MemoryStream(responseData)) as TwitterErrorDetails;
                 }
+#endif
             }
         }
 
+#if !SILVERLIGHT
         /// <summary>
         /// Initializes a new instance of the <see cref="TwitterizerException"/> class.
         /// </summary>
-        /// <param name="info">The object that holds the serialized object data.</param>
-        /// <param name="context">The contextual information about the source or destination.</param>
-        protected TwitterizerException(
-            SerializationInfo info,
-            StreamingContext context)
-            : base(info, context)
+        /// <param name="info">The serialization info.</param>
+        /// <param name="context">The streaming context.</param>
+        protected TwitterizerException(SerializationInfo info, StreamingContext context) :
+            base(info, context)
         {
         }
+#endif
         #endregion
 
         /// <summary>
@@ -156,10 +157,7 @@ namespace Twitterizer
         {
             get
             {
-                if (this.InnerException == null)
-                    return null;
-
-                return ((WebException)this.InnerException).Response;
+                return InnerException == null ? null : ((WebException)this.InnerException).Response;
             }
         }
 
@@ -193,36 +191,13 @@ namespace Twitterizer
                     reportBuilder.AppendFormat(
                         "{0} = \"{1}\"",
                         this.Response.Headers.AllKeys[i],
-                        this.Response.Headers[i]);
+                        this.Response.Headers[this.Response.Headers.AllKeys[i]]);
                 }
 
                 return reportBuilder.ToString();
             }
         }
 
-        #region ISerializable Members
-        /// <summary>
-        /// When overridden in a derived class, sets the <see cref="T:System.Runtime.Serialization.SerializationInfo"/> with information about the exception.
-        /// </summary>
-        /// <param name="info">The <see cref="T:System.Runtime.Serialization.SerializationInfo"/> that holds the serialized object data about the exception being thrown.</param>
-        /// <param name="context">The <see cref="T:System.Runtime.Serialization.StreamingContext"/> that contains contextual information about the source or destination.</param>
-        /// <exception cref="T:System.ArgumentNullException">
-        /// The <paramref name="info"/> parameter is a null reference (Nothing in Visual Basic).
-        /// </exception>
-        /// <PermissionSet>
-        /// <IPermission class="System.Security.Permissions.FileIOPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b77a5c561934e089" version="1" Read="*AllFiles*" PathDiscovery="*AllFiles*"/>
-        /// <IPermission class="System.Security.Permissions.SecurityPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b77a5c561934e089" version="1" Flags="SerializationFormatter"/>
-        /// </PermissionSet>
-        //[SecurityPermissionAttribute(SecurityAction.LinkDemand, SerializationFormatter = true)]
-        public override void GetObjectData(SerializationInfo info, StreamingContext context)
-        {
-            if (info == null)
-                throw new ArgumentNullException("info");
-
-            base.GetObjectData(info, context);
-        }
-        #endregion
-
         /// <summary>
         /// Parses the rate limit headers.
         /// </summary>
@@ -231,20 +206,20 @@ namespace Twitterizer
         {
             this.RateLimiting = new RateLimiting();
 
-            if (!string.IsNullOrEmpty(response.Headers.Get("X-RateLimit-Limit")))
+            if (response.Headers.AllKeys.Contains("X-RateLimit-Limit") && !string.IsNullOrEmpty(response.Headers["X-RateLimit-Limit"]))
             {
-                this.RateLimiting.Total = int.Parse(response.Headers.Get("X-RateLimit-Limit"), CultureInfo.InvariantCulture);
+                this.RateLimiting.Total = int.Parse(response.Headers["X-RateLimit-Limit"], CultureInfo.InvariantCulture);
             }
 
-            if (!string.IsNullOrEmpty(response.Headers.Get("X-RateLimit-Remaining")))
+            if (response.Headers.AllKeys.Contains("X-RateLimit-Remaining") && !string.IsNullOrEmpty(response.Headers["X-RateLimit-Remaining"]))
             {
-                this.RateLimiting.Remaining = int.Parse(response.Headers.Get("X-RateLimit-Remaining"), CultureInfo.InvariantCulture);
+                this.RateLimiting.Remaining = int.Parse(response.Headers["X-RateLimit-Remaining"], CultureInfo.InvariantCulture);
             }
 
-            if (!string.IsNullOrEmpty(response.Headers["X-RateLimit-Reset"]))
+            if (!string.IsNullOrEmpty(response.Headers["X-RateLimit-Reset"]) && !string.IsNullOrEmpty(response.Headers["X-RateLimit-Reset"]))
             {
-                this.RateLimiting.ResetDate = (new DateTime(1970, 1, 1, 0, 0, 0, 0))
-                    .AddSeconds(double.Parse(response.Headers.Get("X-RateLimit-Reset"), CultureInfo.InvariantCulture));
+                this.RateLimiting.ResetDate = DateTime.SpecifyKind(new DateTime(1970, 1, 1, 0, 0, 0, 0)
+                    .AddSeconds(double.Parse(response.Headers["X-RateLimit-Reset"], CultureInfo.InvariantCulture)), DateTimeKind.Utc);
             }
         }
     }
