@@ -26,11 +26,11 @@ namespace Smuxi.Common
 {
     public static class Defines
     {
-        public const string GitBranch   = "master";
-        public const string GitCommitHash = "4136a63";
+        public const string GitBranch   = "release/0.11";
+        public const string GitCommitHash = "24b8867";
 
         private static readonly string f_InstallPrefix = "/usr/local";
-        private static readonly string f_DistVersion = "master/4136a63";
+        private static readonly string f_DistVersion = "release/0.11/24b8867";
         private static readonly string f_TwitterApiKey = "60QV2qQx9cS7y1BJDbgAA|2VgD6qQKddsF5HYQ0TrRgs3tFTnCwDONBmRlTmG658";
 
         public static string InstallPrefix {
