@@ -1,7 +1,7 @@
 /*
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2005-2013 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2005-2014 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -58,6 +58,11 @@ namespace Smuxi.Engine
         DateTime NewsFeedLastModified { get; set; }
         TimeSpan NewsFeedUpdateInterval { get; set; }
         TimeSpan NewsFeedRetryInterval { get; set; }
+        internal MessageBuilderSettings MessageBuilderSettings { get; private set; }
+
+        public event EventHandler<GroupChatPersonAddedEventArgs> GroupChatPersonAdded;
+        public event EventHandler<GroupChatPersonRemovedEventArgs> GroupChatPersonRemoved;
+        public event EventHandler<GroupChatPersonUpdatedEventArgs> GroupChatPersonUpdated;
 
         public IList<IProtocolManager> ProtocolManagers {
             get {
@@ -150,6 +155,8 @@ namespace Smuxi.Engine
             _UserConfig.Changed += OnUserConfigChanged;
             _FilterListController = new FilterListController(_UserConfig);
             _Filters = _FilterListController.GetFilterList().Values;
+            MessageBuilderSettings = new MessageBuilderSettings();
+            MessageBuilderSettings.ApplyConfig(_UserConfig);
             _Chats = new List<ChatModel>();
 
             InitSessionChat();
@@ -183,7 +190,9 @@ namespace Smuxi.Engine
         protected MessageBuilder CreateMessageBuilder()
         {
             var builder = new MessageBuilder();
-            builder.ApplyConfig(UserConfig);
+            // copy settings so the caller can override settings without
+            // changing the settings of the complete session
+            builder.Settings = new MessageBuilderSettings(MessageBuilderSettings);
             return builder;
         }
 
@@ -481,6 +490,7 @@ namespace Smuxi.Engine
                 "config (save|load|list)",
                 "config get key",
                 "config set key=value",
+                "config remove key",
                 "shutdown"
             };
 
@@ -694,11 +704,7 @@ namespace Smuxi.Engine
                 case "get":
                 case "list":
                     string key = null;
-                    if (action == "get") {
-                        if (cd.DataArray.Length < 3) {
-                            _NotEnoughParameters(cd);
-                            return;
-                        }
+                    if (action == "get" && cd.DataArray.Length >= 3) {
                         key = cd.DataArray[2];
                     }
                     foreach (var entry in _UserConfig.OrderBy(kvp => kvp.Key)) {
@@ -717,7 +723,7 @@ namespace Smuxi.Engine
                         _NotEnoughParameters(cd);
                         return;
                     }
-                    string setParam = cd.DataArray[2];
+                    var setParam = String.Join(" ", cd.DataArray.Skip(2).ToArray());
                     if (!setParam.Contains("=")) {
                         builder.AppendErrorText(
                             _("Invalid key/value format.")
@@ -725,34 +731,87 @@ namespace Smuxi.Engine
                         AddMessageToFrontend(cd, builder.ToMessage());
                         return;
                     }
-                    string setKey = setParam.Split('=')[0];
-                    string setValue = setParam.Split('=')[1];
+                    var setKey = setParam.Split('=')[0].Trim();
+                    var setValue = String.Join(
+                        "=", setParam.Split('=').Skip(1).ToArray()
+                    ).Trim();
                     object oldValue = _UserConfig[setKey];
+                    if (oldValue == null && setKey.StartsWith("MessagePatterns/")) {
+                        var id = setKey.Split('/')[1];
+                        var parsedId = Int32.Parse(id);
+                        var msgPatternSettings = new MessagePatternListController(_UserConfig);
+                        var pattern = msgPatternSettings.Get(parsedId);
+                        if (pattern == null) {
+                            // pattern does not exist, create it with default values
+                            pattern = new MessagePatternModel(parsedId);
+                            msgPatternSettings.Add(pattern, parsedId);
+                            oldValue = _UserConfig[setKey];
+                        }
+                    }
                     if (oldValue == null) {
                         builder.AppendErrorText(
                             _("Invalid config key: '{0}'"),
                             setKey
                         );
-                    } else {
-                        try {
-                            object newValue = Convert.ChangeType(setValue, oldValue.GetType());
-                            _UserConfig[setKey] = newValue;
-                            builder.AppendText("{0} = {1}", setKey, newValue.ToString());
-                        } catch (InvalidCastException) {
-                            builder.AppendErrorText(
-                                _("Could not convert config value: '{0}' to type: {1}"),
-                                setValue,
-                                oldValue.GetType().Name
-                            );
-                        } catch (FormatException) {
-                            builder.AppendErrorText(
-                                _("Could not convert config value: '{0}' to type: {1}"),
-                                setValue,
-                                oldValue.GetType().Name
-                            );
+                        AddMessageToFrontend(cd, builder.ToMessage());
+                        return;
+                    }
+
+                    try {
+                        object newValue = Convert.ChangeType(setValue, oldValue.GetType());
+                        _UserConfig[setKey] = newValue;
+                        builder.AppendText("{0} = {1}", setKey, newValue.ToString());
+                        if (setKey.StartsWith("MessagePatterns/")) {
+                            MessageBuilderSettings.ApplyConfig(UserConfig);
                         }
+                    } catch (InvalidCastException) {
+                        builder.AppendErrorText(
+                            _("Could not convert config value: '{0}' to type: {1}"),
+                            setValue,
+                            oldValue.GetType().Name
+                        );
+                    } catch (FormatException) {
+                        builder.AppendErrorText(
+                            _("Could not convert config value: '{0}' to type: {1}"),
+                            setValue,
+                            oldValue.GetType().Name
+                        );
                     }
                     break;
+                case "remove": {
+                    if (cd.DataArray.Length < 3) {
+                        _NotEnoughParameters(cd);
+                        return;
+                    }
+                    var removeParam = cd.DataArray[2];
+                    if (!removeParam.StartsWith("MessagePatterns/")) {
+                        builder.AppendErrorText(
+                            _("Invalid config remove key: '{0}'. Valid remove " +
+                              "keys: MessagePatterns/{{ID}}."),
+                            removeParam
+                        );
+                        AddMessageToFrontend(cd, builder.ToMessage());
+                        return;
+                    }
+                    var id = removeParam.Split('/')[1];
+                    var parsedId = Int32.Parse(id);
+                    var patternController = new MessagePatternListController(_UserConfig);
+                    var pattern = patternController.Get(parsedId);
+                    if (pattern == null) {
+                        builder.AppendErrorText(
+                            _("Message pattern with ID: '{0}' does not exist."),
+                            id
+                        );
+                    } else {
+                        patternController.Remove(parsedId);
+                        MessageBuilderSettings.ApplyConfig(UserConfig);
+                        builder.AppendText(
+                            _("Message pattern with ID: '{0}' removed."),
+                            id
+                        );
+                    }
+                    break;
+                }
                 default:
                     builder.AppendErrorText(
                         _("Invalid parameter for config; use load, save, get or set.")
@@ -766,27 +825,12 @@ namespace Smuxi.Engine
         {
             Trace.Call(cmd);
 
-            FrontendManager frontendMgr = cmd != null ? cmd.FrontendManager : null;
 #if LOG4NET
             f_Logger.Info("Shutting down...");
 #endif
-            lock (_ProtocolManagers) {
-                foreach (var protocolManager in _ProtocolManagers) {
-                    try {
-                        protocolManager.Disconnect(frontendMgr);
-                        protocolManager.Dispose();
-                    } catch (Exception ex) {
-#if LOG4NET
-                        f_Logger.ErrorFormat(
-                            "CommandShutdown(): {0}.Disconnect/Dispose() " +
-                            "failed, continuing with shutdown...",
-                            protocolManager.ToString()
-                        );
-                        f_Logger.Error("CommandShutdown(): Exception", ex);
-#endif
-                    }
-                }
-            }
+
+            var frontendMgr = cmd != null ? cmd.FrontendManager : null;
+            Shutdown(true, frontendMgr);
 
             if (IsLocal) {
                 // allow the frontend to cleanly terminate
@@ -1278,6 +1322,10 @@ namespace Smuxi.Engine
                     fm.AddPersonToGroupChat(groupChat, person);
                 }
             }
+
+            OnGroupChatPersonAdded(
+                new GroupChatPersonAddedEventArgs(groupChat, person)
+            );
         }
         
         public void UpdatePersonInGroupChat(GroupChatModel groupChat, PersonModel oldPerson, PersonModel newPerson)
@@ -1308,6 +1356,10 @@ namespace Smuxi.Engine
                     fm.UpdatePersonInGroupChat(groupChat, oldPerson, newPerson);
                 }
             }
+
+            OnGroupChatPersonUpdated(
+                new GroupChatPersonUpdatedEventArgs(groupChat, oldPerson, newPerson)
+            );
         }
     
         public void UpdateTopicInGroupChat(GroupChatModel groupChat, MessageModel topic)
@@ -1348,6 +1400,10 @@ namespace Smuxi.Engine
                     fm.RemovePersonFromGroupChat(groupChat, person);
                 }
             }
+
+            OnGroupChatPersonRemoved(
+                new GroupChatPersonRemovedEventArgs(groupChat, person)
+            );
         }
         
         public void SetNetworkStatus(string status)
@@ -1620,6 +1676,9 @@ namespace Smuxi.Engine
                 // TODO: use a timeout here to only refresh once in 1 second
                 _Filters = _FilterListController.GetFilterList().Values;
             }
+            if (e.Key == "Interface/Chat/HighlightWords") {
+                MessageBuilderSettings.ApplyConfig(UserConfig);
+            }
             if (e.Key == "Interface/Notebook/EngineBufferLines") {
                 lock (_Chats) {
                     foreach (var chat in _Chats) {
@@ -1651,6 +1710,36 @@ namespace Smuxi.Engine
             UpdatePresenceStatus(newStatus, newMessage);
         }
 
+        public void Shutdown()
+        {
+            Shutdown(false, null);
+        }
+
+        public void Shutdown(bool clean, FrontendManager frontendManager)
+        {
+            Trace.Call(clean, frontendManager);
+
+            lock (_ProtocolManagers) {
+                foreach (var protocolManager in _ProtocolManagers) {
+                    try {
+                        if (clean) {
+                            protocolManager.Disconnect(frontendManager);
+                        }
+                        protocolManager.Dispose();
+                    } catch (Exception ex) {
+#if LOG4NET
+                        f_Logger.ErrorFormat(
+                            "Shutdown(): {0}.Disconnect/Dispose() " +
+                            "failed, continuing with shutdown...",
+                            protocolManager.ToString()
+                        );
+                        f_Logger.Error("Shutdown(): Exception", ex);
+#endif
+                    }
+                }
+            }
+        }
+
         void UpdatePresenceStatus(PresenceStatus status, string message)
         {
             lock (_ProtocolManagers) {
@@ -1827,9 +1916,127 @@ namespace Smuxi.Engine
             }
         }
 
+        protected virtual void OnGroupChatPersonAdded(GroupChatPersonAddedEventArgs e)
+        {
+            if (GroupChatPersonAdded != null) {
+                GroupChatPersonAdded(this, e);
+            }
+
+            var pm = e.GroupChat.ProtocolManager;
+            var hooks = new HookRunner("engine", "session", "on-group-chat-person-added");
+            hooks.Environments.Add(new ChatHookEnvironment(e.GroupChat));
+            if (pm != null) {
+                hooks.Environments.Add(new ProtocolManagerHookEnvironment(pm));
+            }
+            hooks.Environments.Add(new PersonHookEnvironment(e.AddedPerson));
+
+            var cmdChar = (string) UserConfig["Interface/Entry/CommandCharacter"];
+            hooks.Commands.Add(new SessionHookCommand(this, e.GroupChat, cmdChar));
+            if (pm != null) {
+                hooks.Commands.Add(new ProtocolManagerHookCommand(pm, e.GroupChat, cmdChar));
+            }
+
+            // show time
+            hooks.Init();
+            hooks.Run();
+        }
+
+        protected virtual void OnGroupChatPersonRemoved(GroupChatPersonRemovedEventArgs e)
+        {
+            if (GroupChatPersonRemoved != null) {
+                GroupChatPersonRemoved(this, e);
+            }
+
+            var pm = e.GroupChat.ProtocolManager;
+            var hooks = new HookRunner("engine", "session", "on-group-chat-person-removed");
+            hooks.Environments.Add(new ChatHookEnvironment(e.GroupChat));
+            if (pm != null) {
+                hooks.Environments.Add(new ProtocolManagerHookEnvironment(pm));
+            }
+            hooks.Environments.Add(new PersonHookEnvironment(e.RemovedPerson));
+
+            var cmdChar = (string) UserConfig["Interface/Entry/CommandCharacter"];
+            hooks.Commands.Add(new SessionHookCommand(this, e.GroupChat, cmdChar));
+            if (pm != null) {
+                hooks.Commands.Add(new ProtocolManagerHookCommand(pm, e.GroupChat, cmdChar));
+            }
+
+            // show time
+            hooks.Init();
+            hooks.Run();
+        }
+
+        protected virtual void OnGroupChatPersonUpdated(GroupChatPersonUpdatedEventArgs e)
+        {
+            if (GroupChatPersonUpdated != null) {
+                GroupChatPersonUpdated(this, e);
+            }
+
+            var pm = e.GroupChat.ProtocolManager;
+            var hooks = new HookRunner("engine", "session", "on-group-chat-person-updated");
+            hooks.Environments.Add(new ChatHookEnvironment(e.GroupChat));
+            if (pm != null) {
+                hooks.Environments.Add(new ProtocolManagerHookEnvironment(pm));
+            }
+            hooks.Environments.Add(new PersonHookEnvironment("OLD_", e.OldPerson));
+            hooks.Environments.Add(new PersonHookEnvironment("NEW_", e.NewPerson));
+
+            var cmdChar = (string) UserConfig["Interface/Entry/CommandCharacter"];
+            hooks.Commands.Add(new SessionHookCommand(this, e.GroupChat, cmdChar));
+            if (pm != null) {
+                hooks.Commands.Add(new ProtocolManagerHookCommand(pm, e.GroupChat, cmdChar));
+            }
+
+            // show time
+            hooks.Init();
+            hooks.Run();
+        }
+
         private static string _(string msg)
         {
             return LibraryCatalog.GetString(msg, _LibraryTextDomain);
         }
     }
+
+    public abstract class GroupChatEventArgs : EventArgs
+    {
+        public GroupChatModel GroupChat { get; protected set; }
+    }
+
+    public class GroupChatPersonAddedEventArgs : GroupChatEventArgs
+    {
+        public PersonModel AddedPerson { get; private set; }
+
+        public GroupChatPersonAddedEventArgs(GroupChatModel groupChat, PersonModel addedPerson)
+        {
+            GroupChat = groupChat;
+            AddedPerson = addedPerson;
+        }
+    }
+
+    public class GroupChatPersonRemovedEventArgs : GroupChatEventArgs
+    {
+        public PersonModel RemovedPerson { get; private set; }
+
+        public GroupChatPersonRemovedEventArgs(GroupChatModel groupChat, PersonModel removedPerson)
+        {
+            GroupChat = groupChat;
+            RemovedPerson = removedPerson;
+        }
+    }
+
+    public class GroupChatPersonUpdatedEventArgs : GroupChatEventArgs
+    {
+        public PersonModel OldPerson { get; private set; }
+        public PersonModel NewPerson { get; private set; }
+
+        public GroupChatPersonUpdatedEventArgs(GroupChatModel groupChat,
+                                               PersonModel oldPerson,
+                                               PersonModel newPerson)
+        {
+            GroupChat = groupChat;
+            OldPerson = oldPerson;
+            NewPerson = newPerson;
+        }
+    }
 }
