@@ -1,6 +1,6 @@
 // Smuxi - Smart MUltipleXed Irc
 //
-// Copyright (c) 2013 Mirco Bauer <meebey@meebey.net>
+// Copyright (c) 2013-2014 Mirco Bauer <meebey@meebey.net>
 //
 // Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
 //
@@ -97,12 +97,23 @@ namespace Smuxi.Frontend.Gnome
             column.SetCellDataFunc(iconRenderer, new Gtk.TreeCellDataFunc(RenderChatViewIcon));
             AppendColumn(column);
 
-            var cellRenderer = new Gtk.CellRendererText();
+            var cellRenderer = new Gtk.CellRendererText() {
+                Ellipsize = Pango.EllipsizeMode.End
+            };
             column = new Gtk.TreeViewColumn(null, cellRenderer);
             column.Spacing = 0;
+            column.Expand = true;
             column.Sizing = Gtk.TreeViewColumnSizing.Autosize;
             column.SetCellDataFunc(cellRenderer, new Gtk.TreeCellDataFunc(RenderChatViewName));
             AppendColumn(column);
+
+            cellRenderer = new Gtk.CellRendererText();
+            column = new Gtk.TreeViewColumn(null, cellRenderer);
+            column.Spacing = 0;
+            column.Alignment = 1;
+            column.Sizing = Gtk.TreeViewColumnSizing.Autosize;
+            column.SetCellDataFunc(cellRenderer, new Gtk.TreeCellDataFunc(RenderChatViewActivity));
+            AppendColumn(column);
         }
 
         public virtual void Append(ChatView chatView)
@@ -172,7 +183,9 @@ namespace Smuxi.Frontend.Gnome
             }
 
             Gtk.TreePath visibleStart, visibleEnd;
-            GetVisibleRange(out visibleStart, out visibleEnd);
+            if (!GetVisibleRange(out visibleStart, out visibleEnd)) {
+                return false;
+            }
             var chatIter = FindChatIter(chatView);
             var chatPath = TreeStore.GetPath(chatIter);
             // we ignore 0 on purpose, say if a few pixels of a row are returned
@@ -207,6 +220,13 @@ namespace Smuxi.Frontend.Gnome
                                                   Gtk.CellRenderer cellr,
                                                   Gtk.TreeModel model, Gtk.TreeIter iter)
         {
+            if (cellr == null) {
+                throw new ArgumentNullException("cellr");
+            }
+            if (model == null) {
+                throw new ArgumentNullException("model");
+            }
+
             var chat = (ChatView) model.GetValue(iter, 0);
             var renderer = (Gtk.CellRendererPixbuf) cellr;
 
@@ -227,29 +247,28 @@ namespace Smuxi.Frontend.Gnome
                                                   Gtk.CellRenderer cellr,
                                                   Gtk.TreeModel model, Gtk.TreeIter iter)
         {
+            if (cellr == null) {
+                throw new ArgumentNullException("cellr");
+            }
+            if (model == null) {
+                throw new ArgumentNullException("model");
+            }
+
             var chat = (ChatView) model.GetValue(iter, 0);
             var renderer = (Gtk.CellRendererText) cellr;
 
             Gdk.Color color;
-            string text;
             if (chat.HighlightCount > 1) {
                 color = ThemeSettings.HighlightColor;
-                text = String.Format("{0} ({1})",
-                                     chat.Name,
-                                     chat.HighlightCount.ToString());
             } else if (chat.HighlightCount == 1) {
                 color = ThemeSettings.HighlightColor;
-                text = chat.Name;
             } else if (chat.HasActivity) {
                 color = ThemeSettings.ActivityColor;
-                text = chat.Name;
             } else if (chat.HasEvent) {
                 color = ThemeSettings.EventColor;
-                text = chat.Name;
             } else {
                 // no activity
                 color = ThemeSettings.NoActivityColor;
-                text = chat.Name;
             }
 
             var textColor = TextColorTools.GetBestTextColor(
@@ -261,6 +280,47 @@ namespace Smuxi.Frontend.Gnome
             renderer.Markup = String.Format(
                 "<span foreground=\"{0}\">{1}</span>",
                 GLib.Markup.EscapeText(textColor.ToString()),
+                GLib.Markup.EscapeText(chat.Name)
+            );
+        }
+
+        protected virtual void RenderChatViewActivity(Gtk.TreeViewColumn column,
+                                                      Gtk.CellRenderer cellr,
+                                                      Gtk.TreeModel model, Gtk.TreeIter iter)
+        {
+            if (cellr == null) {
+                throw new ArgumentNullException("cellr");
+            }
+            if (model == null) {
+                throw new ArgumentNullException("model");
+            }
+
+            var chat = (ChatView) model.GetValue(iter, 0);
+            var renderer = (Gtk.CellRendererText) cellr;
+
+            Gdk.Color color;
+            string text = null;
+            if (chat.HighlightCount > 1) {
+                color = ThemeSettings.HighlightColor;
+                text = chat.HighlightCount.ToString();
+            } else {
+                // no highlight counter
+                renderer.Markup = String.Empty;
+                return;
+            }
+            if (text == null) {
+                return;
+            }
+
+            var textColor = TextColorTools.GetBestTextColor(
+                ColorConverter.GetTextColor(color),
+                ColorConverter.GetTextColor(
+                Gtk.Rc.GetStyle(this).Base(Gtk.StateType.Normal)
+                ), TextColorContrast.High
+            );
+            renderer.Markup = String.Format(
+                "<span foreground=\"{0}\">({1})</span>",
+                GLib.Markup.EscapeText(textColor.ToString()),
                 GLib.Markup.EscapeText(text)
             );
         }
