@@ -1,6 +1,6 @@
 // Smuxi - Smart MUltipleXed Irc
 //
-// Copyright (c) 2012 Mirco Bauer <meebey@meebey.net>
+// Copyright (c) 2012-2014 Mirco Bauer <meebey@meebey.net>
 //
 // Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
 //
@@ -219,6 +219,15 @@ namespace Smuxi.Frontend.Gnome
                 // do connect as background task as it might take a while
                 ThreadPool.QueueUserWorkItem(delegate {
                     try {
+                        if (Frontend.EngineVersion < new Version(0, 8, 11)) {
+                            // HACK: Smuxi < 0.8.11 used auto serialization for
+                            // ServerModel and thus breaks on unknown fields,
+                            // which we skip by setting this to null, see:
+                            // ServerModel.GetObjectData()
+                            server.ServerID = null;
+                            server.Nickname = null;
+                            server.Realname = null;
+                        }
                         Frontend.Session.Connect(server, Frontend.FrontendManager);
                     } catch (Exception ex) {
                         Frontend.ShowException(Parent, ex);
@@ -247,6 +256,9 @@ namespace Smuxi.Frontend.Gnome
                 
                 controller.AddServer(server);
                 controller.Save();
+
+                // reload server list in join bar
+                JoinWidget.InitNetworks(controller.GetNetworks());
             } catch (InvalidOperationException ex) {
                 Frontend.ShowError(Parent, _("Unable to add server: "), ex);
             } catch (Exception ex) {
